<?PHP

/**
 * $Log: servers.php,v $
 * Revision 1.5  2004/07/27 14:31:14  services
 * - utilisation de la methode depuis le fichier de configuration
 *
 * Revision 1.4  2004/07/06 15:32:43  services
 * o securite :
 *  - PHP_AUTH_USER -> REMOTE_USER
 *
 * Revision 1.3  2004/04/19 12:13:11  seb
 * mise a niveau pour gestion securite
 *
 * Revision 1.2  2004/04/15 08:23:12  seb
 * ajout log
 */
require_once 'ws/classes/generic.php';


/**
 * Errors
 */
define(ERROR_UNKNOWN,              10000);
define(ERROR_ACCES_DENIED,         10001);
define(ERROR_TRACEROUTE_NO_SERVER, 10010);
define(ERROR_TRACEROUTE_NO_ROUTE,  10011);

/**
 * class for all related servers web services. 
 *
 * methods must be called statically. For each method there is
 * a corresponding array wich define the type(public/ client), 
 * method signature and a string doc which describes the aim of 
 * the method.
 *
 * @package ws_eemyle
 */
class wsc_eemyle extends wsc_generic {

    var $get_display_lists = array ( 'sig'  => array(array('struct')),
                                     'doc'  => "Retourne l'ensemble des listes pour afficher les details. Retourne array(-1) en cas d'erreur." );
    
    /**
     * returns display lists
     *
     * @return array
     */
    function &get_display_lists ($params) {
    
        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){

            // SQL stuff
            $manufacturers    = DB_DataObject::factory('manufacturers');
            $os               = DB_DataObject::factory('os');
            $processors       = DB_DataObject::factory('processors');
            $memories         = DB_DataObject::factory('memories');
            $usages           = DB_DataObject::factory('usage_type');

            // Lists
            $display_lists = Array();
            $display_lists['manufacturers']    = Array();
            $display_lists['oses']             = Array();
            $display_lists['processors']       = Array();
            $display_lists['memories']         = Array();
            $display_lists['usages']           = Array();

            // Manufacturers
            $manufacturers->selectAdd();
            $manufacturers->selectAdd('id_manufacturer');
            $manufacturers->selectAdd('name');
            $manufacturers->OrderBy('name');
            $manufacturers->find();

            while ($manufacturers->fetch()){
                array_push($display_lists['manufacturers'], Array($manufacturers->id_manufacturer,
                                                                  $manufacturers->name));
            }

            // Os
            $os->selectAdd();
            $os->selectAdd('id_os');
            $os->selectAdd('name');
            $os->OrderBy('name');
            $os->find();

            while ($os->fetch()){
                array_push($display_lists['oses'], Array($os->id_os,
                                                         $os->name));
            }
            
            // Processors
            $manufacturers = DB_DataObject::factory('manufacturers');
            $processors->joinAdd($manufacturers);
            $processors->selectAdd();
            $processors->selectAdd('id_processor');
            $processors->selectAdd('manufacturers.name AS manufacturer_name');
            $processors->selectAdd('processors.name    AS processor_name');
            $processors->OrderBy('manufacturers.name, processors.name');
            $processors->find();

            while ($processors->fetch()){
                array_push($display_lists['processors'], Array($processors->id_processor,
                                                               $processors->manufacturer_name . ' - ' . $processors->processor_name));
            }

            // Usages
            $usages->selectAdd();
            $usages->selectAdd('id_usage');
            $usages->selectAdd('name');
            $usages->OrderBy('name');
            $usages->find();

            while ($usages->fetch()){
                array_push($display_lists['usages'], Array($usages->id_usage,
                                                           $usages->name));
            }

            return new XML_RPC_Response(XML_RPC_encode($display_lists));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));
        }
    }

    var $get_customers_having_server = array ( 'sig'  => array(array('struct')),
                                               'doc'  => "Retourne la liste des clients ayant un ou plusieur serveur." );

    /**
     * returns detailled customers array 
     *
     * @return array
     */
    function &get_customers_having_server ($params) {

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){

            // SQL stuff
            $company = DB_DataObject::factory('company');
            $servers = DB_DataObject::factory('servers');

            // Getting the customers' list on which the user has access
            $acl_customers = parent::get_acl_service_customer($user, 'eemyle');
            if (is_array($acl_customers)) {
                $company->whereAdd('eeid        IN (\'' . implode('\', \'', $acl_customers) . '\')');
                $servers->whereAdd('id_customer IN (\'' . implode('\', \'', $acl_customers) . '\')');
            }

            /* Fetch all customers having servers */
            $servers->dead = False;  // Only alived server are interesting
            $servers->selectAdd();
            $servers->selectAdd('DISTINCT id_customer');
            $servers->orderBy('id_customer');
            $servers->find();

            $id_customers = array();
            while ($servers->fetch()) {
                array_push($id_customers, $servers->id_customer);
            }
           
            /* Fetching all customers */
            $company->orderBy('eeid');
            $company->find();

            $customers = array();
            while ($company->fetch()) {
                $customer = $company->toArray();

                if (in_array($customer['eeid'], $id_customers)) {
                    $customer['has_server'] = True;
                } else {
                    $customer['has_server'] = False;
                }

                array_push($customers, $customer);
            }

            return new XML_RPC_Response(XML_RPC_encode($customers));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }
    }

    var $get_contacts_by_customer_id = array ( 'sig'  => array(array('struct', 'base64')),
                                               'doc'  => "Retourne la liste des contacts d'aprs l'Eeid du client." );
    
    /**
     * returns contacts array from a given customer id
     *
     * @return array
     */
    function &get_contacts_by_customer_id ($params) {
    
        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){
        
            $arg0        = $params->getParam(0);
            $id_customer = XML_RPC_decode($arg0);

            // SQL stuff
            $company = DB_DataObject::factory('company');
            $contact = DB_DataObject::factory('contact');

            // Getting customer's informations
            $company->eeid = $id_customer;
            $company->find(true);

            $contacts = array();
            array_push($contacts, Array('first_name' => '',
                                        'surname'    => $company->name,
                                        'email'      => '',
                                        'phone'      => $company->phone,
                                        'mobile'     => '',
                                        'fax'        => $company->fax));
            
            // Getting contacts' informations
            $sql_query = "SELECT contact.* FROM contact,
            (    SELECT contact_knot_table.* FROM
            knot_table as contact_knot_table,  knot_table as customer_knot_table,  vcontent_table, company
            WHERE   vcontent_table.tag_id = 18
            AND vcontent_table.content_value = CAST(customer_knot_table.knot_id AS text)
            AND vcontent_table.knot_id = contact_knot_table.knot_id
            AND customer_knot_table.knot_cid = company.knot_cid AND company.eeid = '$id_customer'
            )  AS contact_knot_table
            WHERE  contact.knot_cid = contact_knot_table.knot_cid 
            ORDER BY contact.surname
            " ;
            $contact->query($sql_query);
    
            while($contact->fetch()){
                array_push($contacts, $contact->toArray());
            }

            return new XML_RPC_Response(XML_RPC_encode($contacts));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }

    }

    var $get_servers_by_customer_id = array ( 'sig'  => array(array('struct', 'base64')),
                                              'doc'  => "Retourne la liste des serveurs d'aprs l'Eeid du client." );
    
    /**
     * returns servers array from a given customer id
     *
     * @return array
     */
    function &get_servers_by_customer_id ($params) {
    
        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){
        
            $arg0        = $params->getParam(0);
            $id_customer = XML_RPC_decode($arg0);

            // SQL stuff
            $servers = DB_DataObject::factory('servers');
            $os      = DB_DataObject::factory('os');
            $update  = DB_DataObject::factory('security_update');

            if (! $id_customer == '') {
                $servers->id_customer = $id_customer;
            }

            //$servers->dead = False;
            $servers->joinAdd($os);

            // Getting the customers' list on which the user has access
            $acl_customers = parent::get_acl_service_customer($user, 'eemyle');
            if (is_array($acl_customers)) {
                $servers->whereAdd('id_customer IN (\'' . implode('\', \'', $acl_customers) . '\')');
            }

            $servers->selectAdd();
            $servers->selectAdd('id_server');
            $servers->selectAdd('id_customer');
            $servers->selectAdd('dead');
            $servers->selectAdd('servers.name as server_name');
            $servers->selectAdd('os.name      as os_name');
            $servers->selectAdd('security_check');
            $servers->selectAdd('date(now()) - date(last_eepdate) AS last_eepdate');
            $servers->selectAdd('usage');
            $servers->selectAdd('id_server_physical_host');

            $servers->orderBy('id_customer,server_name');
            $servers->find();
    
            $servers_coll = array();
            while ($servers->fetch()){
                $server = $servers->toArray();

                // The number of package which have to be upgraded
                $update->id_server = $server['id_server'];
                $update->whereAdd('updated=false');
                $server['updates'] = $update->count();

                array_push($servers_coll, $server);
            }

            return new XML_RPC_Response(XML_RPC_encode($servers_coll));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));
        }
    }

    var $get_server_by_server_id = array ( 'sig'  => array(array('struct', 'base64')),
                                           'doc'  => "Retourne les dtails d'un serveur d'aprs son ID." );
    
    /**
     * returns servers array from a given customer id
     *
     * @return array
     */
    function &get_server_by_server_id ($params) {
    
        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){
        
            $arg0      = $params->getParam(0);
            $id_server = XML_RPC_decode($arg0);

            // SQL stuff
            $servers       = DB_DataObject::factory('servers');
            $users         = DB_DataObject::factory('users');
            $usages        = DB_DataObject::factory('usage');
            $interfaces    = DB_DataObject::factory('interfaces');

            $servers->selectAdd('servers.name  AS server_name');
            $servers->selectAdd('date(now()) - date(last_eepdate) AS last_eepdate');
            $servers->whereAdd("id_server = $id_server");

            /* Fetching server informations */
            $n = $servers->find(True);

            $server = $servers->toArray();
            $server['users']                     = Array();
            $server['interfaces']                = Array();
            $server['usages']                    = Array();
            $server['previous_hops_data']        = Array();
            $server['physical_host_data']        = Array();
            $server['connection_interface_data'] = Array();

            /* Fetching users */
            $users->id_server = $id_server;
            $users->find();

            while ($users->fetch()) {
                array_push($server['users'], $users->toArray());
            }

            /* Fetching interfaces */
            $interfaces->id_server = $id_server;
            $interfaces->orderBy('name');
            $interfaces->find();

            while ($interfaces->fetch()) {
                array_push($server['interfaces'], $interfaces->toArray());
                array_push($server['connection_interface_data'], Array($interfaces->id_interface, $interfaces->name . ' (' . $interfaces->ip . ')'));
            }

            /* Fetching usages */
            $usages->id_server = $id_server;
            $usages->find();

            while ($usages->fetch()) {
                array_push($server['usages'], $usages->id_usage);
            }

            /* Fetching previous_hops_data */
            $servers = DB_DataObject::factory('servers');
            $servers->selectAdd();
            $servers->selectAdd('id_server');
            $servers->selectAdd('name');
            $servers->id_customer = $server['id_customer'];
            $servers->dead        = False;
            $servers->whereAdd('NOT id_server = ' . $id_server);
            $servers->orderBy('name');

            /* Fetching servers informations */
            array_push($server['previous_hops_data'], Array(0,  'none'));
            array_push($server['previous_hops_data'], Array(-1, 'cf. comments'));
            $servers->find();
            while ($servers->fetch()) {
                array_push($server['previous_hops_data'], Array($servers->id_server,
                                                                $servers->name));
            }

            /* Fetching physical_host_data */
            $servers = DB_DataObject::factory('servers');
            $servers->selectAdd();
            $servers->selectAdd('id_server');
            $servers->selectAdd('name');
            $servers->id_customer = $server['id_customer'];
            $servers->dead        = False;
            $servers->whereAdd('NOT id_server = ' . $id_server);
            $servers->orderBy('name');

            /* Default physical host setting */
            array_push($server['physical_host_data'], Array(0,  'none'));
            array_push($server['physical_host_data'], Array(-1, 'cf. comments'));
            $servers->find();
            while ($servers->fetch()) {
                array_push($server['physical_host_data'], Array($servers->id_server,
                                                                $servers->name));
            }

            return new XML_RPC_Response(XML_RPC_encode($server));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));
        }
    }

    var $get_server_traceroute_by_server_id = array ( 'sig'  => array(array('struct', 'base64')),
                                                      'doc'  => "Retourne la route pour atteindre un serveur d'aprs son ID." );

    /**
     * returns the traceroute to connect to a server
     *
     * @return array
     */
    function &get_server_traceroute_by_server_id ($params) {

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){
            $max_loop  = 10;
            $arg0      = $params->getParam(0);
            $id_server = XML_RPC_decode($arg0);

            $traceroute = Array();

            $loop = 0;
            while (isset($id_server) and $loop < $max_loop) {
                $loop += 1;

                // SQL stuff
                $servers = DB_DataObject::factory('servers');
    
                $servers->id_server = $id_server;
                $n = $servers->find(True);
                if (! $n == 1) {
                    // No server
                    $loop       = $max_loop;
                    $traceroute = ERROR_TRACEROUTE_NO_SERVER;
                    break;

                } else {
                    /* Try to find a valid interface */
                    $interfaces = DB_DataObject::factory('interfaces');
                    $interfaces->id_interface = $servers->id_connection_interface;
                    $interfaces->selectAdd();
                    $interfaces->selectAdd('ip');

                    $n = $interfaces->find(True);
                    $ip_address = '';
                    if ($n == 0 || $servers->id_server_previous_hop == -1) {
                        // No interface
                        $loop       = $max_loop;
                        $traceroute = ERROR_TRACEROUTE_NO_ROUTE;
                        break;

                    } elseif ($n == 1) {
                        $ip_address = $interfaces->ip;

                    }
                }
                
                // If the server is a backup server, it's name is "hostname (X)", but it's real hostname is "hostname"
                $server_name = split(' ',  $servers->name);
                $server_name = split('\.', $server_name[0]);
                $server_name = $server_name[0];

                /* Fetch the username to use to connect to the server */
                $users = DB_DataObject::factory('users');
                $users->id_user = $servers->id_connection_user;
                $users->find(True);
                $admin_name = $users->name;
                $admin_pass = $users->pass;

                /* Fetch the root user of the server */
                $users = DB_DataObject::factory('users');
                $users->id_server = $id_server;
                $users->name      = 'root';
                $n = $users->find(True);
                if ($n == 1) {
                    $root_pass = $users->pass;
                    $root_name = $users->name;
                } else {
                    /* For switches */
                    $users = DB_DataObject::factory('users');
                    $users->id_server = $id_server;
                    $users->name      = 'enable';
                    $n = $users->find(True);
                    if ($n == 1) {
                        $root_pass = $users->pass;
                        $root_name = $users->name;
                    } else {
                        $root_pass = '';
                    }
                }

                /* Build the route */
                array_unshift($traceroute, Array( 'server_name'  => $server_name,
                                                  'ip'           => $ip_address,
                                                  'ssh_port'     => $servers->ssh_port,
                                                  'id_server'    => $id_server,
                                                  'session_name' => $servers->id_customer . ' - ' . $servers->name,
                                                  'admin_name'   => $admin_name,
                                                  'admin_pass'   => $admin_pass,
                                                  'root_name'    => $root_name,
                                                  'root_pass'    => $root_pass ));

                /* Looping if the server has a previous hop */
                if ($servers->id_server_previous_hop > 0) {
                    $id_server = $servers->id_server_previous_hop;

                } else {
                    unset($id_server);

                }
            }

            /* An error occured */
/*            if ($loop == $max_loop) {
                return new XML_RPC_Response(XML_RPC_encode($traceroute));
            }*/

            return new XML_RPC_Response(XML_RPC_encode($traceroute));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));
        }
    }

    var $get_tickets_by_server_id = array ( 'sig'  => array(array('struct', 'base64')),
                                            'doc'  => "Retourne la liste des tickets de moins d'un an lis  un server." );
     /**
     * return all ticket number relative to the server which Told date > now() - 1 year
     *
     * @param int server id
     * @return array
     */
    function &get_tickets_by_server_id($params) {
        global $OPT;  

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){
            $arg0      = $params->getParam(0);
            $id_server = XML_RPC_decode($arg0);

            // SQL stuff
            require_once 'DB.php';
            $db =& DB::connect($OPT['rt']['db_dsn']);
            if (DB::isError($db)) {
                return new XML_RPC_Response(XML_RPC_encode($db->getMessage()));
            }
            $db->setFetchMode(DB_FETCHMODE_ASSOC);

            $today = getdate();
            $year  = $today['year'] - 1;
            $date  = $year .'-'.
            $today['mon'].'-'.
            $today['mday'];

            $requete = "SELECT Tickets.id, Tickets.Subject, Tickets.Status, Tickets.Told, Tickets.LastUpdated, Users.Name, Users.RealName
                        FROM CustomFields, ObjectCustomFieldValues, Tickets, Users
                        WHERE CustomFields.Name='serveur'
                        AND CustomFields.id=ObjectCustomFieldValues.CustomField
                        AND ObjectCustomFieldValues.Content='$id_server'
                        AND Tickets.id=ObjectCustomFieldValues.ObjectId
                        AND Tickets.Told >= '$date'
                        AND Tickets.Owner = Users.id
                        ORDER BY Tickets.id DESC";

            $res =& $db->query($requete);

            if (DB::isError($res)) {
                return new XML_RPC_Response(XML_RPC_encode($res->getMessage()));
            }

            $tickets = array();
            while ($row =& $res->fetchRow()) {
                $ticket = Array('number'       => $row['id'],
                                'subject'      => $row['subject'],
                                'date'         => $row['told'],
                                'last_updated' => $row['lastupdated'],
                                'status'       => $row['status'],
                                'owner'        => $row['realName'] . ' (' . $row['name'] . ')');

                array_push($tickets, $ticket);
            }

            $db->disconnect();

            return new XML_RPC_Response(XML_RPC_encode($tickets));
        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));
        }
    }

    var $get_link_to_server_by_server_id = array('sig'  => array(array('base64', 'base64')),
                                                 'doc'  => '
base64 : URL

Pour un identifiant serveur donn retourne le lien vers la page (auquel il faut ajouter
 l identifiant) du serveur dcrivant ce serveur.'
                    );

    /**
     * return a full specified link to server details for a given server id
     *
     * @param int server id
     * @return string url to the server details
     */
    function &get_link_to_server_by_server_id($params) {
        global $OPT;    // import user errcode value

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, array('tech', 'com'))){

            $arg0      = $params->getParam(0);
            $id_server = XML_RPC_decode($arg0);

            $server_url = $OPT['eemyle']['url_host'] .
                          $OPT['eemyle']['url_base'] .
                          $OPT['eemyle']['func_server_browse'] .
                          $id_server;
    
            return new XML_RPC_Response(XML_RPC_encode($server_url));
            
        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }
    }

    var $get_link_to_ticket_by_ticket_id = array('sig'  => array(array('base64', 'base64')),
                                                 'doc'  => '
base64 : URL

Pour un identifiant ticket donn retourne le lien vers la page (auquel il faut ajouter
 l identifiant) du serveur dcrivant ce ticket.'
                    );

    /**
     * return a full specified link to ticket details for a given ticket id
     *
     * @param int server id
     * @return string url to the server details
     */
    function &get_link_to_ticket_by_ticket_id($params) {
        global $OPT;    // import user errcode value

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){

            $arg0      = $params->getParam(0);
            $id_ticket = XML_RPC_decode($arg0);

            $server_url = $OPT['rt']['url_ticket'] .
                          $id_ticket;
    
            return new XML_RPC_Response(XML_RPC_encode($server_url));
            
        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }
    }

    /**
     * Servers functions
     *
     */
    var $add_server = array ( 'sig'  => array(array('bool', 'base64', 'string'),
                              'doc'  => "Cre un nouveau serveur pour le client dfinit par id_customer" ));

    function &add_server($params) {

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')) {
        
            $arg0        = $params->getParam(0);
            $id_customer = XML_RPC_decode($arg0);

            $arg1        = $params->getParam(1);
            $server      = XML_RPC_decode($arg1);
            $server      = xmlrpc_decode($server, 'UTF-8');

            // SQL stuff
            $servers = DB_DataObject::factory('servers');

            $servers->id_customer     = $id_customer;
            $servers->id_os           = 4;              // Beurk! this is the id of the "unknown" os
            $servers->name            = 'New server';
            $servers->domain_name     = '';

            foreach($server as $key => $value) {
                $servers->$key = $value;
            }

            $result = $servers->insert();

            return new XML_RPC_Response(XML_RPC_encode($result, 'bool'));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }
    }

    var $remove_server = array ( 'sig'  => array(array('bool', 'base64'),
                                 'doc'  => "Supprime un serveur d'aprs son ID" ));

    function &remove_server($params) {

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')) {
        
            $arg0      = $params->getParam(0);
            $id_server = XML_RPC_decode($arg0);

            // SQL stuff
            $servers = DB_DataObject::factory('servers');

            $servers->id_server = $id_server;
            $n = $servers->find(True);
            if ($n == 1) {
                /* Users */
                $users = DB_DataObject::factory('users');
                $users->id_server = $id_server;
                $result = $users->delete();

                /* Usages */
                if ($result == true || $result == 0) {
                    $usages = DB_DataObject::factory('usage');
                    $usages->id_server = $id_server;
                    $result = $usages->delete();
                }

                /* Interfaces */
                if ($result == true || $result == 0) {
                    $interfaces = DB_DataObject::factory('interfaces');
                    $interfaces->id_server = $id_server;
                    $result = $interfaces->delete();
                }

                /* And finally the server */
                if ($result == true || $result == 0) {
                    $result = $servers->delete();
                }

            } else {
                $result = False;

            }

            return new XML_RPC_Response(XML_RPC_encode($result, 'bool'));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }
    }

    var $duplicate_server = array ( 'sig'  => array(array('bool', 'base64'),
                                    'doc'  => "Duplique un serveur d'aprs son ID" ));

    function &duplicate_server($params) {
        
        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')) {
            
            $arg0      = $params->getParam(0);
            $id_server = XML_RPC_decode($arg0);

            // SQL stuff
            $servers = DB_DataObject::factory('servers');

            $servers->id_server = $id_server;
            $n = $servers->find(True);
            if ($n == 1) {
                /* Duplicate the server */
                $servers->id_server    = '';
                $servers->name         = $servers->name . ' (copy)';
                $servers->last_eepdate = null; // DB_DataObject_Cast::sql('NULL');   // FIXME: null means today ;-)

                $id_new_server = $servers->insert();

                if (! $id_new_server == false) {

                    // SQL stuff
                    $users         = DB_DataObject::factory('users');
                    $usages        = DB_DataObject::factory('usage');
                    $interfaces    = DB_DataObject::factory('interfaces');

                    /* Users */
                    $users->id_server = $id_server;
                    $users->find();
                    while ($users->fetch()) {
                        $new_users = clone($users);
                        $new_users->id_user   = '';
                        $new_users->id_server = $id_new_server;
                        $new_users->insert();
                    }

                    /* Usages */
                    $usages->id_server = $id_server;
                    $usages->find();
                    while ($usages->fetch()) {
                        $new_usages = clone($usages);
                        $new_usages->id_server = $id_new_server;
                        $new_usages->insert();
                    }

                    /* Interfaces */
                    $interfaces->id_server = $id_server;
                    $interfaces->find();
                    while ($interfaces->fetch()) {
                        $new_interfaces = clone($interfaces);
                        $new_interfaces->id_interface = '';
                        $new_interfaces->id_server    = $id_new_server;
                        $new_interfaces->insert();
                    }

                    $result = True;

                } else {
                    $result = True;

                }

            } else {
                $result = False;

            }

            return new XML_RPC_Response(XML_RPC_encode($result, 'bool'));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));
        }
    }

    var $update_server = array ( 'sig'  => array(array('bool', 'base64', 'string'),
                                 'doc'  => "Mise  jour d'un serveur" ));

    function &update_server($params) {

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')) {
            
            $arg0      = $params->getParam(0);
            $id_server = XML_RPC_decode($arg0);

            $arg1      = $params->getParam(1);
            $server    = XML_RPC_decode($arg1);
            $server    = xmlrpc_decode($server, 'UTF-8');

            // SQL stuff
            $servers = DB_DataObject::factory('servers');

            $servers->id_server = $id_server;
            $n = $servers->find(True);
            if ($n == 1) {

                /* Users */
                if (isset($server['users'])) {
                    $users = DB_DataObject::factory('users');

                    $id_users = Array();
                    foreach($server['users'] as $user) {
                        // Insert
                        if ($user['id_user'] == -1) {
                            $users->id_server = $id_server;
                            $users->name      = $user['name'];
                            $users->pass      = $user['pass'];
                            array_push($id_users, $users->insert());

                        // Update
                        } else {
                            $users->id_user = $user['id_user'];
                            $users->name    = $user['name'];
                            $users->pass    = $user['pass'];
                            $users->update();
                            array_push($id_users, $user['id_user']);
                        }
                    }

                    // Remove
                    $users = DB_DataObject::factory('users');
                    $users->whereAdd('id_server = ' . $id_server);
                    $users->whereAdd('NOT id_user IN (' . implode(', ', $id_users) . ')');
                    $users->delete(true);

                    unset($server['users']);

                }

                /* Connection */
                if (isset($server['connectable'])) {
                    if ($server['id_server_previous_hop'] == 0) {
                        $server['id_server_previous_hop']  = '';
                    }

                    if ($server['id_connection_interface'] == -1) {
                        $server['id_connection_interface'] = '';
                    }

                    if ($server['id_connection_user'] == -1) {
                        $server['id_connection_user']      = '';
                    }

                    if ($server['connectable'] == 'f') {
                        // Resetting to default values
                        $server['id_server_previous_hop']  = '';
                        $server['id_connection_user']      = '';
                        $server['id_connection_interface'] = '';
                        $server['ssh_port']                = 22;
                    }
                }

                /* Usages */
                if (isset($server['usages'])) {
                    $usages = DB_DataObject::factory('usage');

                    $usages->id_server = $id_server;
                    $usages->delete();

                    foreach($server['usages'] as $id_usage) {
                        $usages->id_server = $id_server;
                        $usages->id_usage  = $id_usage;
                        $usages->insert();
                    }

                    unset($server['usages']);
                }

                foreach($server as $key => $value) {
                    $servers->$key = $value;
                }
                $result = $servers->update();

            } else {
                $result = False;

            }
 
            return new XML_RPC_Response(XML_RPC_encode($result, 'bool'));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));
        }
    }

    /**
     * Interfaces functions
     *
     */
    var $add_interface = array ( 'sig'  => array(array('bool', 'base64', 'string'),
                                 'doc'  => "Cre une nouvelle interface rseau pour le serveur dfinit par id_server" ));

    function &add_interface($params) {

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')) {

            $arg0      = $params->getParam(0);
            $id_server = XML_RPC_decode($arg0);

            $arg1      = $params->getParam(1);
            $interface = XML_RPC_decode($arg1);
            $interface = xmlrpc_decode($interface, 'UTF-8');

            // SQL stuff
            $interfaces = DB_DataObject::factory('interfaces');
            $interfaces->id_server = $id_server;
            $interfaces->setFrom($interface);

            $result = $interfaces->insert();

            return new XML_RPC_Response(XML_RPC_encode($result, 'bool'));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }
    }

    var $remove_interface = array ( 'sig'  => array(array('bool', 'base64'),
                                    'doc'  => "Suppression d'une interface" ));

    function &remove_interface($params) {
        
        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')) {

            $arg0         = $params->getParam(0);
            $id_interface = XML_RPC_decode($arg0);

            // SQL stuff
            $interfaces = DB_DataObject::factory('interfaces');

            $interfaces->id_interface = $id_interface;
            $n = $interfaces->find(True);
            if ($n == 1) {
                $result = $interfaces->delete();
            } else {
                $result = False;
            }

            return new XML_RPC_Response(XML_RPC_encode($result, 'bool'));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }
    }

    var $update_interface = array ( 'sig'  => array(array('bool', 'base64', 'string')),
                                    'doc'  => "Mise  jour d'une interface" );

    function &update_interface($params) {
        
        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){
            
            $arg0         = $params->getParam(0);
            $id_interface = XML_RPC_decode($arg0);

            $arg1         = $params->getParam(1);
            $interface    = XML_RPC_decode($arg1);
            $interface    = xmlrpc_decode($interface, 'UTF-8');

            // SQL stuff
            $interfaces = DB_DataObject::factory('interfaces');
            $interfaces->id_interface = $id_interface;

            if ($interfaces->find(True) == 1) {
                $interfaces->setFrom($interface);
                $result = $interfaces->update();
            } else {
                $result = False;
            }
    
            return new XML_RPC_Response(XML_RPC_encode($result, 'bool'));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }

    }

    var $get_database_statistics = array ( 'sig'  => array(array('struct')),
                                           'doc'  => "Retourne les statistiques sur la base de donne." );

    /**
     * returns some statistics
     *
     * @return array
     */
    function &get_database_statistics ($params) {

        $user = $_SERVER["REMOTE_USER"];
        if (parent::check_acl_service($user, 'eemyle')){

            // Statistics
            $statistics = Array();
            $statistics['oses_count']   = Array();
            $statistics['usages_count'] = Array();

            /* Number of servers */
            $servers = DB_DataObject::factory('servers');
            $statistics['servers_count'] = $servers->count();

            /* Alive servers */
            $servers = DB_DataObject::factory('servers');
            $servers->dead = 'f';
            $statistics['alive_servers_count'] = $servers->count();

            /* Dead servers */
            $servers = DB_DataObject::factory('servers');
            $servers->dead = 't';
            $statistics['dead_servers_count'] = $servers->count();

            /* Servers with security maintenance */
            $servers = DB_DataObject::factory('servers');
            $servers->dead           = 'f';
            $servers->security_check = 't';
            $statistics['security_servers_count'] = $servers->count();

            /* Connectable servers */
            $servers = DB_DataObject::factory('servers');
            $servers->dead        = 'f';
            $servers->connectable = 't';
            $statistics['connectable_servers_count'] = $servers->count();

            /* Average servers count by customers */
            $servers = DB_DataObject::factory('servers');
            $servers->query('SELECT AVG(count) FROM (SELECT count(*) FROM servers WHERE dead = \'f\' GROUP BY id_customer) AS count');
            $servers->fetch();
            $statistics['avg_servers_per_customer'] = (int) round($servers->avg);

            /* Maximum servers count by customers */
            $servers = DB_DataObject::factory('servers');
            $servers->query('SELECT MAX(count) FROM (SELECT count(*) FROM servers WHERE dead = \'f\' GROUP BY id_customer) AS count');
            $servers->fetch();
            $statistics['max_servers_per_customer'] = $servers->max;

            /* OSes count */
            $servers = DB_DataObject::factory('servers');
            $os      = DB_DataObject::factory('os');

            $servers->joinAdd($os);
            $servers->dead = 'f';
            $servers->selectAdd();
            $servers->selectAdd('os.name  AS os_name');
            $servers->selectAdd('count(*) AS count');
            $servers->groupBy('os_name');
            $servers->find();

            while($servers->fetch()) {
                $statistics['oses_count'][$servers->os_name] = $servers->count;
            }

            /* Usages count */
            $usages      = DB_DataObject::factory('usage');
            $usage_types = DB_DataObject::factory('usage_type');

            $usages->joinAdd($usage_types);
            $usages->selectAdd();
            $usages->selectAdd('usage_type.name as usage_name');
            $usages->selectAdd('count(*) AS count');
            $usages->groupBy('usage_name');
            $usages->find();

            while($usages->fetch()) {
                $statistics['usages_count'][$usages->usage_name] = $usages->count;
            }

            /* Public interface count */
            $interfaces = DB_DataObject::factory('interfaces');
            $interfaces->selectAdd();
            $interfaces->selectAdd('DISTINCT id_server');
            $interfaces->pub = 't';
            $statistics['pub_interfaces_count'] = $interfaces->count();

            return new XML_RPC_Response(XML_RPC_encode($statistics));

        } else {
            return new XML_RPC_Response(XML_RPC_encode(ERROR_ACCES_DENIED));

        }
    }

} // end of class

?>
